import pandas as pd

from pyeph.tools.decorators import (
	translate_params,
	validate_group_by,
	validate_div_by
)

from ..calculator import Calculator


class LaborMarket(Calculator):
	"""
		Obtencion de la tasa de desempleo
	"""
	
	# Constantes Desempleo
	VAR_EPH_ANO = 'ANO4'
	VAR_EPH_TRIM = 'TRIMESTRE'
	VAR_EPH_EDAD = 'CH06'
	VAR_EPH_PONDERADOR = 'PONDERA'
	VAR_EPH_ESTADO = 'ESTADO'

	def __init__(self, 
		eph: pd.DataFrame
	):
		self.eph = eph
		
	@classmethod
	def get_pt_pet(cls, eph, div_by):
		if div_by == "PET":
			return eph[eph[cls.VAR_EPH_EDAD]>=14]
		return eph

	def calculate_grouped_indicator(self, df_denominator, df_numerator, indicator_name):
			denominator = df_denominator.groupby(self.VAR_USER).sum()[[self.VAR_EPH_PONDERADOR]]
			denominator = denominator.rename(columns = {self.VAR_EPH_PONDERADOR: 'DENOMINATOR'} )
			numerator = df_numerator.groupby(self.VAR_USER).sum()[[self.VAR_EPH_PONDERADOR]]
			numerator = numerator.rename(columns = {self.VAR_EPH_PONDERADOR: 'NUMERATOR'} )
			df = pd.merge(numerator, denominator, on = self.VAR_USER)
			df[indicator_name] = ((df['NUMERATOR']/df['DENOMINATOR'])*100).round(1)
			return df
	
	def calculate_indicator(self, df_denominator, df_numerator, indicator_name):
			denominador = df_denominator[self.VAR_EPH_PONDERADOR].sum()
			ocupades = df_numerator[self.VAR_EPH_PONDERADOR].sum()
			tasa = ((ocupades/denominador)*100).round(1)
			_data = {indicator_name:[tasa]}
			df = pd.DataFrame(data = _data)
			return df

	def generate_final_df(self, denominator, numerator,  indicator_name):
		if len(self.VAR_USER):
			df = self.calculate_grouped_indicator( denominator, numerator,  indicator_name)
		else: 
			df = self.calculate_indicator( denominator, numerator,  indicator_name)
		return df[[indicator_name]]
 		

	@translate_params({'agrupar_por': 'group_by', 'div_por': 'div_by'})
	@validate_group_by
	@validate_div_by
	def unemployment(self, group_by=[], div_by="PET"):
		_indicator_unemp_name = 'Tasa de Desempleo'
		base_df = self.prepare_eph(self.eph, group_by)
		# Obtencion del denominador
		unemployment_df = self.get_pt_pet(base_df, div_by).copy()
		# Obtiene poblacion economicamente activa (PEA)
		pea = unemployment_df[(unemployment_df[self.VAR_EPH_ESTADO]==1) | (unemployment_df[self.VAR_EPH_ESTADO]==2)]
		desocupades = unemployment_df[unemployment_df[self.VAR_EPH_ESTADO]==2]
		return self.generate_final_df(pea, desocupades,  _indicator_unemp_name)

	@translate_params({'agrupar_por': 'group_by', 'div_por': 'div_by'})
	@validate_group_by
	@validate_div_by
	def employment(self, group_by=[], div_by="PT"):
		_indicator_emp_name = 'Tasa de Empleo'
		base_df = self.prepare_eph(self.eph, group_by)
		# Obtiene poblacion total (PT)
		employment_df = self.get_pt_pet(base_df, div_by).copy()
		ocupades = employment_df[employment_df[self.VAR_EPH_ESTADO]==1]
		return self.generate_final_df(employment_df, ocupades, _indicator_emp_name)

	@translate_params({'agrupar_por': 'group_by', 'div_por': 'div_by'})
	@validate_group_by
	@validate_div_by
	def activity(self, group_by=[], div_by="PT"):
		_indicator_act_name = 'Tasa de Actividad'
		base_df = self.prepare_eph(self.eph, group_by)
		activity_df = self.get_pt_pet(base_df, div_by).copy()
		pea = activity_df[(activity_df[self.VAR_EPH_ESTADO]==1) | (activity_df[self.VAR_EPH_ESTADO]==2)]
		return self.generate_final_df(activity_df, pea,  _indicator_act_name)


	# Traduccion
	desempleo = unemployment
	empleo = employment
	actividad = activity
# Traduccion
MercadoLaboral = LaborMarket


QUARTER_OR_WAVE_OPTION = """Para bases de datos hasta 2003, el tipo de frecuencia (freq) debe ser 'onda'
Para bases de datos desde 2004, el tipo de frecuencia (freq) debe ser 'trimestre'"""

INDEC_2007_3 = """INDEC advierte: La informacion correspondiente al tercer trimestre 
2007 no esta disponible ya que los aglomerados Mar del Plata-Batan, 
Bahia Blanca-Cerri y Gran La Plata no fueron relevados por causas 
de orden administrativo, mientras que los datos correspondientes al 
Aglomerado Gran Buenos Aires no fueron relevados por paro del 
personal de la EPH."""

INDEC_STATS_EMERGENCY = """En el marco de la emergencia estadistica el INDEC no publico la base solicitada. 
Mas informacion en: https://www.indec.gob.ar/ftp/cuadros/sociedad/anexo_informe_eph_23_08_16.pdf"""

INDEC_2007_2015 = """INDEC advierte:
Advertencia sobre el uso de series estadisticas. Se advierte que las series estadisticas publicadas con
posterioridad a enero del 2007 y hasta diciembre 2015 deben ser consideradas con reservas,
excepto las que ya hayan sido revisadas en 2016 y su difusion lo consigne expresamente.
El INDEC, en el marco de las atribuciones conferidas por los decretos 181/15 y 55/16, dispuso las investigaciones
requeridas para establecer la regularidad de procedimientos de obtencion de datos, su procesamiento, elaboracion de indicadores y difusion
Mas informacion en: https://www.indec.gob.ar/ftp/cuadros/sociedad/anexo_informe_eph_23_08_16.pdf"""



import os
import pandas as pd
from pandas.api.types import is_numeric_dtype


from .. import config

LABELS_FILENAME = 'EPH_tot_urbano_estructura_bases.xlsx'

def get_df():
    pathfile = os.path.join(config.ROOT_DIR, config.FILES_DIR, LABELS_FILENAME)
    df = pd.read_excel(pathfile, sheet_name = 'BASE PERSONAS', skiprows=6, engine='openpyxl').dropna(axis=0, how='all')
    df = df.fillna(method='ffill')
    df['CAMPO'] = df['CAMPO'].str.strip()
    df['DESCRIPCIÓN'] = df['DESCRIPCIÓN'].str.replace(r'([0-9]\.)', '=', regex=True)
    df[['CODIGO', 'DESCRIPCIÓN']] = df['DESCRIPCIÓN'].str.split("=",expand=True,)
    return df

def vars_labels():
    labels = get_df()
    labels = labels[labels['DESCRIPCIÓN'].isna()][['CAMPO', 'CODIGO']].rename(columns = {'CODIGO': 'DESCRIPCION'})
    return labels

def map_labels(df):
    #Obtenemos las columnas relevantes para etiquetar
    df = df.reset_index()
    cols = [c for c in df.columns]
    # Obtenemos dataframe de labels
    labels = get_df()
    labels = labels[labels['DESCRIPCIÓN'].notna()]
    labels['DESCRIPCIÓN'] = labels['DESCRIPCIÓN'].astype(str)
    labels = labels[labels['CAMPO'].isin(cols)]
    variables = labels['CAMPO'].unique()


    for i in variables:            
        # Obtener la serie que mapea para cada variable
        df_i = labels[labels['CAMPO'] == i].rename(columns = {'CODIGO': f'{i}', 'DESCRIPCIÓN': f'DESCRIPCIÓN_{i}'}).reset_index()
        
        if i=='AGLOMERADO': #codigo en el excel le falta un 0 adelante para que sea comparable con el codigo de eph. se agrega el 0
            df_i[i] = df_i[i].str.replace(r"\D(\d)$", r" 0\1")

        df_i[i] = df_i[i].str.strip()
        df_i = df_i.drop(columns = {'CAMPO', 'TIPO (longitud)', 'index'}).set_index(i).rename(columns = {f'DESCRIPCIÓN_{i}': i})
        dict_i = df_i.to_dict().get(i)

        if is_numeric_dtype(df[i]):
            df[i] = df[i].astype(int).astype(str)
        df[i] = df[i].astype(str).str.strip()
        df[i] = df[i].map(dict_i)
        

    # Renombrar columnas con su etiqueta
    var_names = vars_labels()
    var_names = var_names[var_names['CAMPO'].isin(cols)]
    var_names_dict = var_names.set_index('CAMPO').to_dict().get('DESCRIPCION')
    return df.rename(columns = var_names_dict)


# Traduccion
etiquetas_vars = vars_labels
etiquetar = map_labels


import sys

from pyeph.errors import *
from pyeph.ads import (
	INDEC_2007_2015,
	INDEC_2007_3,
	INDEC_STATS_EMERGENCY,
	QUARTER_OR_WAVE_OPTION
)

from .types import Frequency

class MicroDataValidator:
	"""
		Validacion de datos para la consulta de las bases
	"""

	def __init__(self):
		self.handle_exceptions_warnings()

	def valid_period_freq(self):
		if self.period > Frequency.VALUES[self.freq]:
			raise ValueError("La frecuencia solicitada ({}) requiere un periodo menor o igual a {}".format(self.freq, Frequency.VALUES[self.freq]))

	def valid_year_freq(self):
		if self.year <= 2003 and self.freq != 'onda': 
			raise ValueError(QUARTER_OR_WAVE_OPTION)
		if self.year >= 2004 and self.freq != 'trimestre': 
			raise ValueError(QUARTER_OR_WAVE_OPTION)			

	def valid_existent_db(self):
		if self.year==2007 and self.period==3:
			raise NonExistentDBError(INDEC_2007_3)

		if (self.year == 2015 and self.period in [3,4]) or (self.year==2016 and self.period==1):
			raise NonExistentDBError(INDEC_STATS_EMERGENCY)

		if self.year >= 2007 and self.year <= 2015:
			sys.stdout.write(INDEC_2007_2015)
			
	def handle_exceptions_warnings(self):
		self.valid_period_freq()
		self.valid_year_freq()
		self.valid_existent_db()



import pandas as pd

from pyeph.tools.decorators import (
    validate_group_by,
    translate_params
    )

from pyeph.get.basket import Basket
from pyeph.get.equivalent_adult import EquivalentAdult

from ..types import BasketType
from ..calculator import Calculator


class Poverty(Calculator):
    """
		Obtencion de tasa de pobreza e indigencia de hogar y de personas.
	"""

    # Constantes Pobreza
    VAR_EPH_ANO = 'ANO4'
    VAR_EPH_TRIM = 'TRIMESTRE'
    VAR_EPH_SEXO = 'CH04'
    VAR_EPH_EDAD = 'CH06'
    VAR_EPH_HOGAR = 'CODUSU'
    VAR_EPH_NRO_HOGAR = 'NRO_HOGAR'
    VAR_EPH_REGION = 'REGION'
    VAR_EPH_ITF = 'ITF'

    VAR_CBT = 'CBT'
    VAR_CBA = 'CBA'
    VAR_CBT_HOGAR = 'CBT_HOGAR'
    VAR_CBA_HOGAR = 'CBA_HOGAR'
    VAR_AE = 'adequi'
    VAR_AE_HOGAR = 'adequi_hogar'
    VAR_PERIODO = 'periodo'
    VAR_BASKET_COD = 'codigo'
    VAR_STATUS = 'situacion'
    VAR_INDICATOR = 'Indicador'
    VAR_POVERTY = 'Pobreza'
    VAR_INDIGENCE = 'Indigencia'

    # No cambiar el orden de la lista de status
    STATUS_OPTIONS = ['indigente', 'pobre', 'no pobre']

    basket = BasketType() # Salva de recibir algo != a pandas.DataFrame

    @translate_params({'canasta': 'basket'})
    def __init__(self, 
        eph: pd.DataFrame, 
        basket: pd.DataFrame=None
    ):
        self.eph = eph
        check_pond = [c for c in self.eph.columns if c == 'PONDIH']

        if len(check_pond):
            Poverty.VAR_EPH_PONDERADOR_INGRESO = 'PONDIH'

        else:
            Poverty.VAR_EPH_PONDERADOR_INGRESO = 'PONDERA'

        
        # En caso de que el usuario no especifique una canasta, se descarga la del repo.
        if basket is None:
            self.basket = Basket().get_df().copy()
        else:
            self.basket = self.prepare_basket(basket.copy())
        # Obtencion del archivo con los valores del adultx equivalente
        self.equivalent_adult = EquivalentAdult().get_df().copy()

    @classmethod
    def compute_status(cls, row):
        if row[cls.VAR_EPH_ITF] is None:
            return None
        if row[cls.VAR_EPH_ITF] < row[cls.VAR_CBA_HOGAR]:
            return cls.STATUS_OPTIONS[0]
        if row[cls.VAR_EPH_ITF] < row[cls.VAR_CBT_HOGAR]:
            return cls.STATUS_OPTIONS[1]
        return cls.STATUS_OPTIONS[2]

    @classmethod
    def get_pivot_df(cls, df, calc_type, drop_level=True):
        df = df[df.index.get_level_values(cls.VAR_STATUS).isin(cls.STATUS_OPTIONS[:2])].reset_index()
        df = df.rename(columns={cls.VAR_STATUS: cls.VAR_INDICATOR, cls.VAR_EPH_PONDERADOR_INGRESO: 'Valor'})
        new_name_poverty = '{} ({})'.format(cls.VAR_POVERTY, calc_type)
        new_name_indigence = '{} ({})'.format(cls.VAR_INDIGENCE, calc_type)
        df[cls.VAR_INDICATOR] = df[cls.VAR_INDICATOR].replace([cls.STATUS_OPTIONS[0], cls.STATUS_OPTIONS[1]],
                                                                [new_name_indigence, new_name_poverty])
        if len(cls.VAR_USER) >= 1 and drop_level:
            df = pd.pivot_table(df, index=cls.VAR_USER, columns=cls.VAR_INDICATOR).round(3)
            df.columns = df.columns.droplevel()
        else:
            df = pd.pivot_table(df, columns=cls.VAR_INDICATOR).round(3)
        df[new_name_poverty] = df[new_name_poverty] + df[new_name_indigence]
        return df

    def generate_base_df(self, eph, basket, equivalent_adult):
        # Agrega variable periodo a la base de eph
        eph = eph.reset_index().drop(columns="index")
        period = self.get_period(eph)
        eph.loc[:, (self.VAR_PERIODO)] = period
        # Se obtiene los valores de las canastas para el periodo especificado
        basket = basket[self.basket[self.VAR_PERIODO] == period]
        basket = basket.groupby(by=[self.VAR_PERIODO, self.VAR_BASKET_COD]).mean().reset_index()
        basket = basket.drop(columns={self.VAR_PERIODO})
        # Se mergea con los valores de la CBT y CBA para es período
        eph = eph.merge(basket, how="left", left_on=self.VAR_EPH_REGION, right_on=self.VAR_BASKET_COD)
        # Se mergea con la información del adultx equivalente
        eph = eph.merge(equivalent_adult, how="left", on=[self.VAR_EPH_SEXO, self.VAR_EPH_EDAD])
        # Calculo de la CBT y CBA por adultx equivalente hogar
        eph[self.VAR_AE_HOGAR] = eph.groupby([self.VAR_EPH_HOGAR, self.VAR_EPH_NRO_HOGAR, self.VAR_PERIODO])[
            self.VAR_AE].transform('sum')
        eph[self.VAR_CBT_HOGAR] = eph[self.VAR_CBT] * eph[self.VAR_AE_HOGAR]
        eph[self.VAR_CBA_HOGAR] = eph[self.VAR_CBA] * eph[self.VAR_AE_HOGAR]
        # Calculo de status (indigente, pobre, no pobre)
        eph[self.VAR_STATUS] = eph.apply(self.compute_status, axis=1)
        return eph

    @translate_params({'agrupar_por': "group_by"})
    @validate_group_by
    def population(self, group_by=[]):
        # Calculo de pobreza e indigencia (poblacion)
        eph = self.prepare_eph(self.eph, group_by)
        base_df = self.generate_base_df(eph, self.basket, self.equivalent_adult)
        population_rate = self.distribution(base_df, self.VAR_EPH_PONDERADOR_INGRESO,
                                            self.VAR_USER + [self.VAR_STATUS])
        population_rate = self.get_pivot_df(population_rate, '%')
        population_quantity = base_df.groupby(by=self.VAR_USER + [self.VAR_STATUS])[
            self.VAR_EPH_PONDERADOR_INGRESO].sum()
        population_quantity = self.get_pivot_df(population_quantity, '#')
        df = population_rate.join(population_quantity)
        df = df.sort_index(ascending=False, axis=1)
        return df
         

    def household(self):
        # Calculo de pobreza e indigencia (hogares)
        eph = self.prepare_eph(self.eph, [])
        base_df = self.generate_base_df(eph, self.basket, self.equivalent_adult)
        household_df = base_df.copy()
        household_df = household_df[
            [self.VAR_EPH_HOGAR, self.VAR_EPH_PONDERADOR_INGRESO, self.VAR_STATUS]].drop_duplicates()
        household_rate = self.distribution(household_df, self.VAR_EPH_PONDERADOR_INGRESO, [self.VAR_STATUS])
        household_rate = self.get_pivot_df(household_rate, '%', False)
        household_quantity = household_df.groupby(by=self.VAR_STATUS)[self.VAR_EPH_PONDERADOR_INGRESO].sum()
        household_quantity = self.get_pivot_df(household_quantity, '#', False)
        df = household_rate.join(household_quantity)
        df = df.sort_index(ascending=False, axis=1)
        return df

    # Traduccion
    poblacion = population
    hogares = household
# Traduccion
Pobreza = Poverty



import os
import sys
import zipfile
import wget
import pandas as pd

from pyeph.errors import NonExistentDBError

MODULE_PATH = os.getcwd()


class Getter:
    """
	Obtencion de archivos para su posterior tratamiento 
	Utiliza los atributos de cada tipo de obtencion declarado en el archivo __init__.py

    Retorno en .get_file()
    -------
        file : str
			archivo .csv ubicado dentro del .zip con el mismo nombre

    Retorno en .get_df()
    -------
        df : pandas.DataFrame
			dataframe del .csv solicitado
	"""

    BASE_GITHUB_URL = "https://github.com/CocinaDeSoftware/pyeph-data/raw/master/"
    URL_INDEC = "https://www.indec.gob.ar/ftp/cuadros/menusuperior/eph"
    DEFAULT_DIR = "pyeph/.db"

    @property
    def download_base_folder(self):
        # ruta/completa/db/
        return os.path.join(MODULE_PATH, self.DEFAULT_DIR)

    @property
    def download_type_folder(self):
        # ruta/completa/db/tipo
        return os.path.join(self.download_base_folder, self.folder)

    @property
    def download_destination(self):
        # ruta/completa/db/tipo/archivo
        return os.path.join(self.download_type_folder, self.filename)

    def from_indec(self):
        pass

    def from_github(self):
        pathfile = os.path.join(self.BASE_GITHUB_URL, self.folder, self.filename).replace('\\', '/')
        wget.download(pathfile, self.download_destination)

    def download(self):
        try:
            self.from_github()
            sys.stdout.write("\n")
        except:
            raise NonExistentDBError()

    def check_if_exists(self):
        # Se fija si existe en el directorio. Caso contrario lo descarga
        base_folder = self.download_base_folder
        if not os.path.exists(base_folder):
            os.makedirs(base_folder)
        type_folder = self.download_type_folder
        if not os.path.exists(type_folder):
            os.makedirs(type_folder)
        zip_file = self.download_destination
        if not os.path.exists(zip_file):
            self.download()
        return zip_file

    def get_file(self):
        # Aquí hay que agregar un progress bar
        zip_file = self.check_if_exists()
        zip_file = zipfile.ZipFile(zip_file, 'r')
        csv_file = zip_file.open(self.filename.replace('.zip', '.csv'), 'r')
        return csv_file

    def get_df(self, inform_user=True):
        """
			Retorna el DataFrame
		"""
        df = pd.read_csv(self.get_file(), low_memory=False)
        if inform_user:
            sys.stdout.write("Obtenido con exito: {} \n".format(self.filename))
        return df



from pyeph.tools.decorators import translate_params

from ..getter import Getter

from .types import (
	Year,
	Period,
	BaseType
)


class Mautic(Getter):
	"""
	Módulo de Acceso y Uso de Tecnologías de la Información y la Comunicación (Encuesta Permanente de Hogares)

    Parametros
    ----------
        year : str
            año de la eph
        period : list
            periodo que se desea consultar
	"""

	PREFIX_FOLDER = "mautic"

	year = Year()
	period = Period()
	base_type = BaseType()

	@translate_params({
		'ano': 'year',
		'periodo': 'period',
		'tipo_base': 'base_type'		
	})
	def __init__(self,
			year: int,
			period: int = 4,
			base_type: str = "individual"
		):
		self.year = year
		self.period = period
		self.base_type = base_type
		super(Mautic, self).__init__()

	@property
	def filename(self): return "mautic_{}_{}T{}.zip".format(
			self.base_type,
			self.year,
			self.period
			)

	@property
	def folder(self): return "{}_{}".format(
			self.PREFIX_FOLDER, 
			self.base_type
			)


import pandas as pd

class EPHType:

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		if not isinstance(value,pd.DataFrame):
			raise ValueError("eph debe ser un pd.DataFrame")
		self.value = value

class BasketType:

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		if not isinstance(value,pd.DataFrame):
			raise ValueError("canasta debe ser un pd.DataFrame")
		self.value = value



from pyeph.get.basket import Basket

from .types import EPHType

class Calculator:
	"""
		Metodos genericos para la utilizacion en mas de un calculo
	"""

	# Constantes Canastas
	BASKET_COLUMNS = ['periodo', 'region', 'CBA', 'CBT', 'codigo']

	eph = EPHType() # Salva de recibir algo != a pandas.DataFrame

	@classmethod
	def prepare_eph(cls, eph, group_by):
		# Variables fundamentales de la EPH
		VAR_FUNDAMENTALS = cls.get_fundamentals()
		if isinstance(group_by, str):
			group_by = [group_by]
		cls.VAR_USER = group_by
		# Filtro de eph para trabajar sobre un df mas pequeño
		VAR_KEEP = VAR_FUNDAMENTALS + cls.VAR_USER
		VAR_KEEP = list(set(VAR_KEEP))
		result = eph[VAR_KEEP].copy()
		return result

	@classmethod
	def prepare_basket(cls, basket):
		if not all(c in basket.columns for c in cls.BASKET_COLUMNS):
			basket = Basket.prepare_basket(basket)
		return basket

	@classmethod
	def info(cls): pass

	@classmethod
	def help(cls): return cls.__doc__.strip()

	@classmethod
	def get_fundamentals(cls):
		return [v for k, v in cls.__dict__.items() if 'VAR_EPH_' in k]

	@classmethod
	def distribution(cls, df, weight_var, values_var):
		df = df.copy()
		if not isinstance(values_var, list):
			values_var = [values_var]
		if len(values_var)>1:
			weights = df.groupby(values_var)[weight_var].sum()
			total_weight = weights.groupby(values_var[:-1]).sum()
			return weights.div(total_weight, axis = 'index')
		values_var = values_var[0]
		weights = df[weight_var]
		total_weight = weights.sum()
		# para el test: check.groupby(values_var[:-1]).sum() == 1.0
		return df.groupby(values_var)[weight_var].sum() / total_weight

	@classmethod
	def get_period(cls, df):
		year = str(df[cls.VAR_EPH_ANO].drop_duplicates()[0])
		trimester = str(df[cls.VAR_EPH_TRIM].drop_duplicates()[0])
		return '.'.join([year,trimester])

# Traduccion
Calculadora = Calculator


import pandas as pd

from pyeph.tools.decorators import translate_params
from pyeph.tools.labels import vars_labels

from ..getter import Getter
from .validator import MicroDataValidator
from .types import (
	Year,
	Period,
	Frequency,
	BaseType
)


class MicroData(Getter, MicroDataValidator):
	"""
	Obtencion del df de la eph solicitada

    Parametros
    ----------
        year : str
            año de la eph
        freq : str
            tipo de frecuencia "trimestral" u "onda" 
        period : list
            periodo que se desea consultar
		base_type : str, optional
			tipo de base a solicitar "individual", "hogar"
	"""
	
	year = Year()
	period = Period()
	freq = Frequency()
	base_type = BaseType()

	@translate_params({
		'ano': 'year',
		'periodo': 'period',
		'frecuencia': 'freq',
		'tipo_base': 'base_type'
	})
	def __init__(self,
			year: int,
			period: int,
			freq: str = "trimestre",
			base_type: str = "individual",
		):
		self.year = year
		self.period = period
		self.freq = freq
		self.base_type = self.folder = base_type
		super(MicroData, self).__init__()

	@property
	def filename(self): return "base_{}_{}{}{}.zip".format(
			self.base_type,
			self.year,
			self.freq[0].upper(),
			self.period
			)

	def get_df(self, inform_user=True):
		pd.DataFrame.help = vars_labels
		return super().get_df(inform_user)


from pyeph.calc.calculator import Calculator


class Template(Calculator):
    """
        Template para agregar nuevos calculos.

		Notas:
			Para crear un nuevo calculo es necesario copiar la carpeta "template", pegar y renombrar
			Se deben agregar constantes como atributos de las clases de cada calculo
			Estos atributos serán las columnas necesarias para el calculo que se pretende agregar
			con "VAR_EPH_nombrecolumna" (ver VAR_EPH_EJEMPLO mas abajo)
			Esto es necesario para que el nuevo calculo no trabaje sobre el DataFrame completo
			sino mas bien sobre las columnas necesarias
    """
    # Constantes EPH

    VAR_EPH_EJEMPLO = 'EJEMPLO'

    def __init__(self, eph):
        self.eph = eph
    
    def run(self): return

    # Traduccion
    correr = run
# Traduccion
Plantilla = Template


from pyeph.errors import NonExistentDBError

from .microdata import MicroData
from .basket import Basket
from .equivalent_adult import EquivalentAdult
from .mautic import Mautic

def get(data: str, *args, **kwargs):
	handles = {
		'microdata': MicroData,
		'eph': MicroData,
		'basket': Basket,
		'canastas': Basket,
		'adulto-equivalente': EquivalentAdult,
		'equivalent-adult': EquivalentAdult,
		'mautic': Mautic
	}

	try:
		db_getter = handles[data](*args, **kwargs)
	except KeyError:
		posibles = ", ".join(handles.keys())
		raise NonExistentDBError("Debe seleccionar un tipo de base posible: [{}]".format(posibles))
	return db_getter.get_df()
# Traduccion
obtener = get


from datetime import date

class BaseType:

	VALUES = ['individual','hogar']

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		if not (isinstance(value,str) and value in self.VALUES):
			raise ValueError("Por favor ingresa un tipo de base valido: " + ", ".join(self.VALUES))
		self.value = value


class Year:

	INITIAL_YEAR = 2016

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		current_year = date.today().year
		if not (isinstance(value,int) and self.INITIAL_YEAR <= value <= current_year):
			raise ValueError("El año debe ser un número entre {} y {}.".format(self.INITIAL_YEAR, current_year))
		self.value = value



class Period:

	VALUES = [4]

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		if not (
			(isinstance(value,int) and value in self.VALUES) or
			value is None
		):
			raise ValueError("Por favor ingresa un numero de trimeste valido: " + ",".join(map(str, self.VALUES)))
		self.value = value




import sys
from datetime import date
from dateutil.relativedelta import relativedelta
import pandas as pd

from ..getter import Getter

class Basket(Getter):
	"""
		Obtencion del df de la canasta basica
	"""

	folder = "canastas"
	filename = "canastas.zip"

	@staticmethod
	def prepare_basket(df):
		df['indice_tiempo'] =  pd.to_datetime(df['indice_tiempo'])
		df['year'] = df["indice_tiempo"].dt.strftime("%Y")
		df['trim'] = ((df['indice_tiempo'].dt.month-1)//3+1).astype(str)
		df['periodo'] = df['year'] + '.' + df['trim']

		_id_vars = ['indice_tiempo', 'tipo_canasta', 'year', 'trim', 'periodo']
		_value_vars = [c for c in df.columns if c not in _id_vars]

		df =pd.melt(df, id_vars=_id_vars, value_vars=_value_vars, var_name='region', value_name='valor')
		_index = [c for c in df.columns if c not in ['valor', 'tipo_canasta']]
		df = pd.pivot_table(df, values='valor', columns='tipo_canasta', index = _index ,aggfunc='sum' )
		df = df.reset_index()
		region_map = {
			"cuyo": 42,
			"gran_buenos_aires":1,
			"noreste":41,
			"noroeste":40,
			"pampeana":43,
			"patagonia":44
		}
		df['codigo'] = df['region'].map(region_map)
		return df		

	def make_filenames(self, year_month):
		year_month = year_month.strftime('%Y-%m')
		return ['cbt_{}.zip'.format(year_month), 'cba_{}.zip'.format(year_month)]

	def get_df(self, inform_user=True):
		"""
			Retorna el DataFrame
		"""
		df_inicial = pd.DataFrame()
		year_month = date.today()
		query = False
		while query == False:
			try:
				# year_month = date(2021, 6, 1) #.today()
				for f in self.make_filenames(year_month):
					self.filename = f
					df_f =  pd.read_csv(self.get_file(), low_memory=False)
					df_f['tipo_canasta'] = f[:3].upper()
					df_inicial = pd.concat([df_inicial, df_f])
				query = True
			except:
				year_month = year_month - relativedelta(months=1)
		
		if inform_user:
			sys.stdout.write("CBT y CBA mas actualizada que se obtuvo: {} \n".format(year_month.strftime('%Y-%m')))
		df_final = self.prepare_basket(df_inicial)
		
		return df_final



from datetime import date

class BaseType:

	VALUES = ['individual','hogar']

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		if not (isinstance(value,str) and value in self.VALUES):
			raise ValueError("Por favor ingresa un tipo de base valido: " + ", ".join(self.VALUES))
		self.value = value

class Frequency:

	VALUES = {
        'trimestre': 4,
        'onda': 2
    }

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		if not (isinstance(value,str) and value in self.VALUES):
			raise ValueError("Por favor ingresa un tipo de base valido: " + ", ".join(self.VALUES.keys()))
		self.value = value

class Period:

	VALUES = [1,2,3,4]

	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		if not (
			(isinstance(value,int) and value in self.VALUES) or
			value is None
		):
			raise ValueError("Por favor ingresa un numero de trimeste valido: " + ",".join(map(str, self.VALUES)))
		self.value = value

class Year:
	def __get__(self, obj, *args):
		return self.value

	def __set__(self, obj, value):
		current_year = date.today().year
		if not (isinstance(value,int) and value <= current_year):
			raise ValueError("El año debe ser un número menor o igual a {}.".format(current_year))
		self.value = value




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyeph',
 'pyeph.calc',
 'pyeph.calc.labor_market',
 'pyeph.calc.poverty',
 'pyeph.calc.template',
 'pyeph.get',
 'pyeph.get.basket',
 'pyeph.get.equivalent_adult',
 'pyeph.get.mautic',
 'pyeph.get.microdata',
 'pyeph.tools']

package_data = \
{'': ['*'], 'pyeph': ['.files/*'], 'pyeph.tools': ['.examples/*']}

install_requires = \
['ipykernel>=6.13.0,<7.0.0',
 'openpyxl>=3.0.10,<4.0.0',
 'pandas>=1.1.5,<2.0.0',
 'wget>=3.2,<4.0',
 'xlrd>=2.0.1,<3.0.0']

setup_kwargs = {
    'name': 'pyeph',
    'version': '1.1.6',
    'description': "PyEPH es una librería para el procesamiento de la Encuesta Permanente de Hogares (eph) en Python. Permite la descarga de archivos de EPH's y otros como la canasta basica y adulto equivalente , como asi también algunos calculos rápidos relacionados con las mismas",
    'long_description': '# PyEPH - Libreria para el procesamiento de la Encuesta Permanente de Hogares en Python \n<a><img src=\'docs/_static/logo.png\' align="right" height="250" /></a>\n\n![PyPI](https://img.shields.io/pypi/v/pyeph?color=orange&)\n![PyPI - License](https://img.shields.io/pypi/l/pyeph?color=purple&)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/pyeph?)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/pyeph?)\n[![Downloads](https://static.pepy.tech/personalized-badge/pyeph?period=total&units=none&left_color=grey&right_color=yellowgreen&left_text=downloads)](https://pepy.tech/project/pyeph)\n\nLa librería Pyeph tiene como objetivo facilitar el procesamiento en Python de las [Encuesta Permanente de Hogares (eph)](https://www.indec.gob.ar/indec/web/Institucional-Indec-BasesDeDatos) publicadas por INDEC de forma periódica. Está pensada como un espacio donde se nuclean y centralizan los cálculos vinculados a las mismas para posteriormente ser utilizadas en investigaciones, artículos, publicaciones, etc.\nEs una librería que hace principal hincapié en la transparencia metodológica utilizando licencias de código abierto y que promueve la colaboración de las comunidades de cientístas de datos, sociales, investigadorxs, desarrolladorxs, periodistas y demás curiosxs.\n\nPermite la descarga de archivos de `EPH\'s` y otros como la `canasta basica` y `adulto equivalente` , como asi también algunos calculos rápidos relacionados con las mismas\n\n# Instalación\n\nPueden probar nuestra notebook de ejemplo en Google Colab\n\n<a href="https://colab.research.google.com/github/institutohumai/pyeph/blob/main/examples.ipynb" target="_blank"> <img src=\'https://colab.research.google.com/assets/colab-badge.svg\' /> </a>\n\nRecordá abrir en una nueva pestaña\n\n### Prerequisitos\n- [Python 3](https://www.python.org/)\n- [pip](https://www.pypi.org/)\n### Instalando PyEPH\n\n- Abra una terminal del sistema y escriba \n\n```bash\n$ pip install pyeph\n```\n\n# Uso básico\n\nLos siguientes son algunos ejemplos de uso. Para ver todos los cálculos podés ir para la documentación\n\nEn inglés\n\n```python\nimport pyeph\n\n# Obtención\neph = pyeph.get(data="eph", year=2021, period=2, base_type=\'individual\') # EPH individual\nbasket = pyeph.get(data="canastas") # canasta basica total y alimentaria\nadequi = pyeph.get(data="adulto-equivalente") # adulto equivalente\n\n# Cálculos de ejemplo de pobreza \npoverty = pyeph.Poverty(eph, basket)\npopulation_poverty = poverty.population(group_by=\'CH04\') # Población pobre por sexo \nlabeled_poverty = pyeph.map_labels(population_poverty) # Etiquetado de las variables\n\n# Cálculos de Mercado Laboral\nlabor_market = pyeph.LaborMarket(eph)\nunemployment = labor_market.unemployment(group_by="REGION", div_by="PT") # Desempleo agrupado por region y dividiendo por Población Total\nlabeled_unemployment = pyeph.map_labels(unemployment) # Etiquetado de las variables\n```\n\nEn español\n\n```python\nimport pyeph\n\n# Obtención\neph = pyeph.obtener(data="eph", ano=2021, periodo=2, tipo_base=\'individual\') # EPH individual\ncanastas = pyeph.obtener(data="canastas") # canasta basica total y alimentaria\nadequi = pyeph.obtener(data="adulto-equivalente") # adulto equivalente\n\n# Cálculos de ejemplo de pobreza \npobreza = pyeph.Pobreza(eph, canastas)\npoblacion_pobre = pobreza.poblacion(agrupar_por=\'CH04\') # Población pobre por sexo \npoblacion_pobre_etiquetado = pyeph.etiquetar(poblacion_pobre) # Etiquetado de las variables\n\n# Cálculos de Mercado Laboral\nmercado_laboral = pyeph.MercadoLaboral(eph)\ndesempleo = mercado_laboral.desempleo(agrupar_por="REGION", div_por="PT") # Desempleo agrupado por region y dividiendo por Población Total\ndesempleo_etiquetado = pyeph.etiquetar(desempleo) # Etiquetado de las variables\n```\n\n# Documentación\n\n[Link del sitio de la documentación](https://pyeph.readthedocs.io/es/latest/) \n\n---\n\n### Tenga en cuenta\n\nEsta librería se encuentra en estado permanente de desarrollo.\n\n> Cualquier colaboración es bienvenida\n\n\n## Agradecimientos\n\nDejamos aquí un especial agradecimiento al equipo de desarrollo de la librería [EPH en R](https://holatam.github.io/eph/authors.html). Todo el amor para elles ❤️ y a  [Rami Argañaraz](https://www.linkedin.com/in/ramiro-arga%C3%B1araz-57764a16b/) por armarnos el loguito 😻 \n\n---\n⌨️ con ❤️\n\n',
    'author': 'Maria Carolina Trogliero, Mariano Valdez Anopa, Maria Gaska',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/institutohumai/pyeph/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



def translate_params(params={}):
    def function_in_question(fn):
        def wrapper(*args, **kwargs):
            for param in params.keys():
                if param in kwargs.keys():
                    kwargs[params[param]] = kwargs.pop(param)
            return fn(*args, **kwargs)
        return wrapper
    return function_in_question

def validate_group_by(fn):
    def wrapper(*args, **kwargs):
        if "group_by" in kwargs.keys():
            if not isinstance(kwargs['group_by'], (list, str)):
                raise TypeError("group_by debe ser una lista o un string")
        return fn(*args, **kwargs)
    return wrapper

def validate_div_by(fn):
    def wrapper(*args, **kwargs):
        options = {'PT': '"PT" (Población Total)', 'PET': '"PET" (Población en Edad de Trabajar)'}
        if "div_by" in kwargs.keys():
            if not (
                isinstance(kwargs['div_by'], str) and
                kwargs['div_by'] in options.keys()
            ):
                raise TypeError("div_by debe ser un string: {}".format(" o ".join(options.values())))
        return fn(*args, **kwargs)
    return wrapper




class BaseError(Exception):
	"""
		Base Error
	"""
	def __init__(self, message=None):
		self.message = message or self.__doc__
		super(BaseError, self).__init__(self.message.strip())


class NonExistentDBError(BaseError):
	"""
		La base solicitada no fue provista por INDEC.
	"""



import pandas as pd

def merge(eph, modulo, report=True):

    _relevant = ['CODUSU', 'ANO4', 'TRIMESTRE', 'NRO_HOGAR']

    if 'COMPONENTE' in eph.columns:
        _relevant.append('COMPONENTE')

    _share = [c for c in modulo if c in eph]

    _drop = [c for c in _share if c not in _relevant]

    modulo = modulo.drop(columns = _drop)

    df = pd.merge(eph, modulo, how= 'left', on=_relevant)

    return df

# Traduccion
aparear = merge